#
#   This script serves to generate fig. 1 from paper doi.org/10.1088/1361-6595/adc7d8
#   Generated by Maximilian Klich
#   last tested 8.04.2025 on python version 3.13.2
#

import os
import matplotlib
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
from matplotlib.patches import Patch
from matplotlib.lines import Line2D
from PIL import Image
from matplotlib.offsetbox import AnchoredText
from matplotlib.patches import Polygon

plt.style.use('bmh')
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True
plt.rcParams['axes.grid.axis'] = 'both'
plt.rcParams['axes.grid.which'] = 'major'
#
plt.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})
matplotlib.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})


##############################################
print("loading density data ...")
data_nqn = np.loadtxt("data/12MHz_200V_densities.txt", skiprows=1).T
data_qne = np.loadtxt("data/28MHz_220V_densities.txt", skiprows=1).T
data_con = np.loadtxt("data/34MHz_400V_densities.txt", skiprows=1).T

##############################################
print("processing density data ...")

order = np.floor(np.log10(np.abs(np.amax(data_qne)))).astype(int)
ni_nqn = np.amax(data_nqn[2,:]*10.0**(-order))*1.1
ni_qne = np.amax(data_qne[2,:]*10.0**(-order))*1.1
ni_con = np.amax(data_con[2,:]*10.0**(-order))*1.1

##############################################
print("plotting density data ...")
fig = plt.figure(figsize=(12.5,4.5))
pgrid = plt.GridSpec(1, 3, hspace=0.32, wspace=0.28)
plt.gcf().subplots_adjust(bottom=0.16, left=0.07, right=0.93, top = 0.88)
##############################################
##############################################
ax = fig.add_subplot(pgrid[0,0])
ax.plot(data_nqn[0,:]*1e3, data_nqn[1,:]*10.0**(-order), lw=2, c='tab:blue', ls='-', label=r'$\bar{n}_\mathrm{e}$')
ax.plot(data_nqn[0,:]*1e3, data_nqn[2,:]*10.0**(-order), lw=2, c='tab:red', ls='-', label=r'$\bar{n}_\mathrm{i,tot}$')
#
ax.set_xlim(0.0, data_nqn[0,-1]*1e3)
ax.set_ylim(0.0, ni_nqn)
#
ax.set_xlabel(r'z [mm]', labelpad=7.0)
ax.set_ylabel(r'$\bar{{n}}_s$ [$10^{{{expo}}} \mathrm{{\,m}}^{{-3}}$]'.format(expo=order), labelpad=7.0)
ax.set_yticks([0, 0.1, 0.2, 0.3])
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{I}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.4)
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{non-neutral mode}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.03), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.5)
ax.add_artist(at)
### add the markers for the discharge modes
polygon_points = [(0,0), (0,ni_nqn), (1,ni_nqn), (1,0)]
polygon = Polygon(polygon_points, closed=True, color='tab:blue', alpha=0.3)
ax.add_patch(polygon)
##############################################
##############################################
ax = fig.add_subplot(pgrid[0,1])
ax.plot(data_qne[0,:]*1e3, data_qne[1,:]*10.0**(-order), lw=2, c='tab:blue', ls='-', label=r'$\bar{n}_\mathrm{e}$')
ax.plot(data_qne[0,:]*1e3, data_qne[2,:]*10.0**(-order), lw=2, c='tab:red', ls='-', label=r'$\bar{n}_\mathrm{i,tot}$')
#
ax.set_xlim(0.0, data_qne[0,-1]*1e3)
ax.set_ylim(0.0, ni_qne)
#
ax.set_xlabel(r'z [mm]', labelpad=7.0)
ax.set_yticks([0,1,2,3])
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{II}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.4)
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{quasi-neutral mode}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.03), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
# at.patch.set_boxstyle("Square,pad=0.")
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.5)
ax.add_artist(at)
### add the markers for the discharge modes
polygon_points = [(0,0), (0,ni_qne), (1,ni_qne), (1,0)]
polygon = Polygon(polygon_points, closed=True, color='tab:red', alpha=0.3)
ax.add_patch(polygon)
##############################################
##############################################
ax = fig.add_subplot(pgrid[0,2])
ax.plot(data_con[0,:]*1e3, data_con[1,:]*10.0**(-order), lw=2, c='tab:blue', ls='-', label=r'$\bar{n}_\mathrm{e}$', zorder=3)
ax.plot(data_con[0,:]*1e3, data_con[2,:]*10.0**(-order), lw=2, c='tab:red', ls='-', label=r'$\bar{n}_\mathrm{i,tot}$', zorder=3)
#
ax.set_xlim(0.0, data_con[0,-1]*1e3)
ax.set_ylim(0.0, ni_con)
#
ax.set_xlabel(r'z [mm]', labelpad=7.0)
# ax.set_yticks([0,100,200,300])
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{III}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16), zorder=2)
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:green')
at.patch.set_alpha(0.4)
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{constricted mode}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.03), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
# at.patch.set_boxstyle("Square,pad=0.")
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:green')
at.patch.set_alpha(0.5)
ax.add_artist(at)
### add the markers for the discharge modes
polygon_points = [(0,0), (0,ni_con), (1,ni_con), (1,0)]
polygon = Polygon(polygon_points, closed=True, color='tab:green', alpha=0.3)
ax.add_patch(polygon)
####
#### manually add the axis for the entire plot
####
# get handles and labels
handles, labels = ax.get_legend_handles_labels()
# create a new axis
leg_ax = fig.add_axes([0.95, 0.2, 0.04, 0.6], frameon=False)
leg_ax.set_xlim(0, 1)
leg_ax.set_ylim(0, 1)
leg_ax.axis('off')
#manually draw the legend
leg_ax.plot([0.5, 0.5], [0.8, 0.7], marker='none', color='tab:blue', ls='-', lw=2)
leg_ax.text(0.4, 0.62, labels[0], verticalalignment='center', rotation=270)
leg_ax.plot([0.5, 0.5], [0.45, 0.35], marker='none', color='tab:red', ls='-', lw=2)
leg_ax.text(0.4, 0.23, labels[1], verticalalignment='center', rotation=270)
# draw the box around the legend
from matplotlib.patches import FancyBboxPatch
bbox = FancyBboxPatch((0, 0), 1, 1, boxstyle="round, pad=0., rounding_size=0.15", 
    edgecolor='none', facecolor='lightgrey')
leg_ax.add_patch(bbox)
##############################################
##############################################
plt.savefig("fig_1.jpg",format="JPEG",dpi=300)
# plt.show()
