#
#   This script serves to generate fig. 10 from paper doi.org/10.1088/1361-6595/adc7d8
#   Generated by Maximilian Klich
#   last tested 8.04.2025 on python version 3.13.2
#

import os
import matplotlib
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
import matplotlib.patches as patches
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import Normalize
from matplotlib.patches import Patch
from matplotlib.lines import Line2D
from PIL import Image
from matplotlib.offsetbox import AnchoredText
from matplotlib.patches import Polygon
from scipy.ndimage import gaussian_filter

plt.style.use('bmh')
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True
plt.rcParams['axes.grid.axis'] = 'both'
plt.rcParams['axes.grid.which'] = 'major'
#
plt.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})
matplotlib.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})

class MidpointNormalize(Normalize):
    def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
        self.midpoint = midpoint
        Normalize.__init__(self, vmin, vmax, clip)

    def __call__(self, value, clip=None):
        # I'm ignoring masked values and all kinds of edge cases to make a
        # simple example...
        x, y = [self.vmin, self.midpoint, self.vmax], [0, 0.5, 1]
        return np.ma.masked_array(np.interp(value, x, y))

##############################################
# data taken from 220V frequency variation

print("loading data ...")
#data 12 MHz #
Se_Pe_12 = np.loadtxt('data/12MHz/ionization_rate_N2_Penning_XT.txt').T
Se_N2_12 = np.loadtxt('data/12MHz/ionization_rate_N2_XT.txt').T
grid_12 = np.loadtxt('data/12MHz/gridcells.txt', skiprows=1)
time_12 = np.loadtxt('data/12MHz/time.txt', skiprows=1)
# data 18 MHz #
Se_Pe_18 = np.loadtxt('data/18MHz/ionization_rate_N2_Penning_XT.txt').T
Se_N2_18 = np.loadtxt('data/18MHz/ionization_rate_N2_XT.txt').T
grid_18 = np.loadtxt('data/18MHz/gridcells.txt', skiprows=1)
time_18 = np.loadtxt('data/18MHz/time.txt', skiprows=1)
#data 34 MHz #
Se_Pe_34 = np.loadtxt('data/34MHz/ionization_rate_N2_Penning_XT.txt').T
Se_N2_34 = np.loadtxt('data/34MHz/ionization_rate_N2_XT.txt').T
grid_34 = np.loadtxt('data/34MHz/gridcells.txt', skiprows=1)
time_34 = np.loadtxt('data/34MHz/time.txt', skiprows=1)

print("processing data ...")
#normalize the time to the periode length
time_12 /= time_12[-1]
time_18 /= time_18[-1]
time_34 /= time_34[-1]

#normalize rho to gain control over the notation
order_12 = np.floor(np.log10(np.abs(np.amax(Se_N2_12)))).astype(int)
Se_N2_12 = Se_N2_12*10.0**(-order_12)
Se_Pe_12 = Se_Pe_12*10.0**(-order_12)
#
order_18 = np.floor(np.log10(np.abs(np.amax(Se_N2_18)))).astype(int)
Se_N2_18 = Se_N2_18*10.0**(-order_12)
Se_Pe_18 = Se_Pe_18*10.0**(-order_18)
#
order_34 = np.floor(np.log10(np.abs(np.amax(Se_N2_34)))).astype(int)
Se_N2_34 = Se_N2_34*10.0**(-order_34)
Se_Pe_34 = Se_Pe_34*10.0**(-order_34)

#filter the data to improve the statistics
ion1_12 = gaussian_filter(Se_N2_12, 1.3, order=0)
ion2_12 = gaussian_filter(Se_Pe_12, 1.3, order=0)
ion1_18 = gaussian_filter(Se_N2_18, 1.3, order=0)
ion2_18 = gaussian_filter(Se_Pe_18, 1.3, order=0)
ion1_34 = gaussian_filter(Se_N2_34, 1.3, order=0)
ion2_34 = gaussian_filter(Se_Pe_34, 1.3, order=0)

##############################################
print("plotting electron data ...")
fig = plt.figure(figsize=(13.5,8))
pgrid = plt.GridSpec(2, 3, hspace=0.3, wspace=0.32)
plt.gcf().subplots_adjust(bottom=0.16, left=0.1, right=0.98, top = 0.94)
#
##############################################
# Se_N2_12 #
##############################################
ax = fig.add_subplot(pgrid[0,0])
#
map = ax.contourf(time_12, grid_12*1e3, ion1_12, 400, cmap=plt.cm.turbo, extend='both')
## mark the times for the snapshots ##
ax.plot([time_12[650], time_12[650]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_12[650], time_12[650]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.67, 0.16, r'$t_{1,a}$', c='w')
ax.plot([time_12[850], time_12[850]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_12[850], time_12[850]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.88, 0.16, r'$t_{2,a}$', c='w')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('bottom')
    A.set_horizontalalignment('center')
    A.set_y(0.9)
    A.set_rotation(70)
## add exponent
ax.text(.95, 1.19, r'$\times 10^{{{expo}}}$'.format(expo=order_12), transform=ax.transAxes, c='k')
##
# ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{a)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{non-neutral mode}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.4), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.5)
ax.add_artist(at)
##############################################
# Se_Pe_12 #
##############################################
ax = fig.add_subplot(pgrid[1,0])
#
map = ax.contourf(time_12, grid_12*1e3, ion2_12, 400, cmap=plt.cm.turbo, extend='both')
## mark the times for the snapshots ##
ax.plot([time_12[650], time_12[650]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_12[650], time_12[650]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.67, 0.16, r'$t_{1,a}$', c='w')
ax.plot([time_12[850], time_12[850]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_12[850], time_12[850]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.88, 0.16, r'$t_{2,a}$', c='w')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('bottom')
    A.set_horizontalalignment('center')
    A.set_y(0.9)
    A.set_rotation(70)
## add exponent
ax.text(.95, 1.19, r'$\times 10^{{{expo}}}$'.format(expo=order_12), transform=ax.transAxes, c='k')
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{b)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##############################################
##############################################
##############################################
##############################################
# Se_N2_18 #
##############################################
ax = fig.add_subplot(pgrid[0,1])
#
map = ax.contourf(time_18, grid_18*1e3, ion1_18, 400, cmap=plt.cm.turbo, extend='both')
## mark the times for the snapshots ##
ax.plot([time_18[480], time_18[480]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[480], time_18[480]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.51, 0.84, r'$t_{1,b}$', c='w')
ax.plot([time_18[800], time_18[800]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[800], time_18[800]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.84, 0.14, r'$t_{2,b}$', c='w')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('bottom')
    A.set_horizontalalignment('center')
    A.set_y(0.9)
    A.set_rotation(70)
## add exponent
ax.text(.95, 1.19, r'$\times 10^{{{expo}}}$'.format(expo=order_12), transform=ax.transAxes, c='k')
##
# ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{c)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
at = AnchoredText(r'\textbf{transition}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.4), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:purple')
at.patch.set_alpha(0.5)
ax.add_artist(at)
##############################################
# Se_Pe_18 #
##############################################
ax = fig.add_subplot(pgrid[1,1])
#
map = ax.contourf(time_18, grid_18*1e3, ion2_18, 400, cmap=plt.cm.turbo, extend='both')
## mark the times for the snapshots ##
ax.plot([time_18[480], time_18[480]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[480], time_18[480]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.51, 0.84, r'$t_{1,b}$', c='w')
ax.plot([time_18[800], time_18[800]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[800], time_18[800]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.84, 0.14, r'$t_{2,b}$', c='w')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('bottom')
    A.set_horizontalalignment('center')
    A.set_y(0.9)
    A.set_rotation(70)
## add exponent
ax.text(.95, 1.19, r'$\times 10^{{{expo}}}$'.format(expo=order_18), transform=ax.transAxes, c='k')
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{d)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##############################################
##############################################
##############################################
##############################################
# Se_N2_34 #
##############################################
ax = fig.add_subplot(pgrid[0,2])
#
map = ax.contourf(time_34, grid_34*1e3, ion1_34, 400, cmap=plt.cm.turbo, extend='both')
## mark the times for the snapshots ##
ax.plot([time_34[450], time_34[450]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_34[450], time_34[450]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.49, 0.85, r'$t_{1,c}$', c='w')
ax.plot([time_18[750], time_18[750]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[750], time_18[750]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.78, 0.11, r'$t_{2,c}$', c='w')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('bottom')
    A.set_horizontalalignment('center')
    A.set_y(0.9)
    A.set_rotation(70)
## add exponent
ax.text(.95, 1.19, r'$\times 10^{{{expo}}}$'.format(expo=order_34), transform=ax.transAxes, c='k')
## add row label
at = AnchoredText(r'$S_\mathrm{i,N_2^{\ +},d}\ [\mathrm{\frac{m^{-3}}{s}}]$', frameon=True, loc='center left',#
 bbox_to_anchor=(1.07,.5), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17, rotation=270))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##
# ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{e)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{quasi-neutral mode}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.4), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.5)
ax.add_artist(at)
##############################################
# Se_Pe_34 #
##############################################
ax = fig.add_subplot(pgrid[1,2])
#
map = ax.contourf(time_34, grid_34*1e3, ion2_34, 400, cmap=plt.cm.turbo, extend='both')
## mark the times for the snapshots ##
ax.plot([time_34[450], time_34[450]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_34[450], time_34[450]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.49, 0.85, r'$t_{1,c}$', c='w')
ax.plot([time_18[750], time_18[750]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[750], time_18[750]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.78, 0.11, r'$t_{2,c}$', c='w')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('bottom')
    A.set_horizontalalignment('center')
    A.set_y(0.9)
    A.set_rotation(70)
## add exponent
ax.text(.95, 1.19, r'$\times 10^{{{expo}}}$'.format(expo=order_34), transform=ax.transAxes, c='k')
## add row label
at = AnchoredText(r'$S_\mathrm{i,N_2^{\ +},P}\ [\mathrm{\frac{m^{-3}}{s}}]$', frameon=True, loc='center left',#
 bbox_to_anchor=(1.07,.5), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17, rotation=270))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{f)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##############################################
plt.savefig("fig_10.jpg",format="JPEG",bbox_inches='tight',dpi=300)
# plt.show()
plt.close()
