#
#   This script serves to generate fig. 11 from paper doi.org/10.1088/1361-6595/adc7d8
#   Generated by Maximilian Klich
#   last tested 8.04.2025 on python version 3.13.2
#

import os
import matplotlib
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
import matplotlib.patches as patches
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import Normalize
from matplotlib.patches import Patch
from matplotlib.lines import Line2D
from PIL import Image
from matplotlib.offsetbox import AnchoredText
from matplotlib.patches import Polygon
from scipy.ndimage import gaussian_filter

plt.style.use('bmh')
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True
plt.rcParams['axes.grid.axis'] = 'both'
plt.rcParams['axes.grid.which'] = 'major'
#
plt.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})
matplotlib.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})

class MidpointNormalize(Normalize):
    def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
        self.midpoint = midpoint
        Normalize.__init__(self, vmin, vmax, clip)

    def __call__(self, value, clip=None):
        # I'm ignoring masked values and all kinds of edge cases to make a
        # simple example...
        x, y = [self.vmin, self.midpoint, self.vmax], [0, 0.5, 1]
        return np.ma.masked_array(np.interp(value, x, y))

##############################################
print("loading data ...")
ne = np.loadtxt('data/ne_XT.txt').T
nHe = np.loadtxt('data/ni_He+_XT.txt').T
nH2 = np.loadtxt('data/ni_He2+_XT.txt').T
nN2 = np.loadtxt('data/ni_N2+_XT.txt').T
nN4 = np.loadtxt('data/ni_N4+_XT.txt').T
field = np.loadtxt('data/E_XT.txt').T
grid = np.loadtxt('data/gridcells.txt', skiprows=1)
time = np.loadtxt('data/time.txt', skiprows=1)

print("processing data ...")
ni = nHe + nH2 + nN2 + nN4
rho = 1.6e-19 * (ni - ne)

#average the ion data - time-dependency is negligible
ni_ave = np.trapezoid(ni, x=time, dx=time[1], axis=1) / time[-1]

#normalize the time to the periode length
time /= time[-1]

#normalize rho to gain control over the notation
order = np.floor(np.log10(np.abs(np.amax(ni_ave)))).astype(int)

E_max = np.amax(np.abs(field*1e-6))

#crop the XT-data
rho_new = rho[0:751,:]

for x in range(0,751):
    for t in range(0,1000):
        if(rho_new[x,t] >= 0.7):
            rho_new[x,t] = 0.7

#filter the data for better resolution
rho_new = gaussian_filter(rho_new, 0.4, order=0)
field_filtered = gaussian_filter(field*1e-3, 1, order=0)
field_filtered2 = gaussian_filter(field*1e-6, 0.5, order=0)

##############################################
print("plotting electron data ...")
fig = plt.figure(figsize=(12.5,4.9))
pgrid = plt.GridSpec(2, 3, hspace=0.37, wspace=0.34)
plt.gcf().subplots_adjust(bottom=0.16, left=0.1, right=0.98, top = 0.94)
#
##############################################
# rho
##############################################
ax = fig.add_subplot(pgrid[:,0])
norm = MidpointNormalize(midpoint=0)
map = ax.contourf(time, grid[0:751]*1e3, rho_new, 400, norm=norm, cmap=plt.cm.bwr, extend='both')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94, label=r'$\rho$ [$\frac{\mathrm{As}}{\mathrm{m^{3}}}$]')
cbar.ax.xaxis.set_label_coords(0.5, 5.1)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_y(2.3)
    A.set_rotation(70)
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{a)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## mark the zoomed zones
rect1 = patches.Rectangle((time[35],0), time[331]-time[35], grid[51]*1e3, linewidth=2, edgecolor='k', facecolor='none')
ax.add_patch(rect1)
ax.text(time[120], grid[55]*1e3, r'\textbf{z\textsubscript{1}}', fontsize=13, color='k', ha='left', va='bottom')
rect2 = patches.Rectangle((time[670], grid[54]*1e3), time[961]-time[670], (grid[91]-grid[54])*1e3, linewidth=2, edgecolor='k', facecolor='none')
ax.add_patch(rect2)
ax.text(time[740], grid[95]*1e3, r'\textbf{z\textsubscript{2}}', fontsize=13, color='k', ha='left', va='bottom')
## add mode label
at = AnchoredText(r'\textbf{III}', frameon=True, loc='upper left',#
 bbox_to_anchor=(-0.23,1.36), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:green')
at.patch.set_alpha(0.4)
ax.add_artist(at)
##############################################
# E #
##############################################
norm = MidpointNormalize(midpoint=0)
ax = fig.add_subplot(pgrid[:,1])
#
map = ax.contourf(time, grid*1e3, field*1e-6, 400, cmap=plt.cm.bwr, norm=norm, extend='both')
ax.contour(time, grid*1e3, field_filtered2, 14, colors="k", linewidths=0.2, linestyles='dashed')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94, label=r'$E_z$ [$\frac{\mathrm{MV}}{\mathrm{m}}$]')
cbar.ax.xaxis.set_label_coords(0.5, 5.1)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_y(2.3)
    A.set_rotation(70)
##
ax.set_ylim(0,0.5)
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{b)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
# at.patch.set_boxstyle("Square,pad=0.")
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## mark the zoomed zones
rect1 = patches.Rectangle((time[35],0), time[331]-time[35], grid[51]*1e3, linewidth=2, edgecolor='k', facecolor='none')
ax.add_patch(rect1)
ax.text(time[120], grid[55]*1e3, r'\textbf{z\textsubscript{1}}', fontsize=13, color='k', ha='left', va='bottom')
rect2 = patches.Rectangle((time[670], grid[54]*1e3), time[961]-time[670], (grid[91]-grid[54])*1e3, linewidth=2, edgecolor='k', facecolor='none')
ax.add_patch(rect2)
ax.text(time[740], grid[95]*1e3, r'\textbf{z\textsubscript{2}}', fontsize=13, color='k', ha='left', va='bottom')
##############################################
# E zone 1#
##############################################
norm = MidpointNormalize(midpoint=0)
ax = fig.add_subplot(pgrid[0,2])
#
map = ax.contourf(time[35:331], grid[0:51]*1e6, field_filtered[0:51,35:331], 400, cmap=plt.cm.bwr, norm=norm, extend='both')
ax.contour(time[35:331], grid[0:51]*1e6, field_filtered[0:51,35:331], 11, colors="k", linewidths=0.2, linestyles='dashed')
## create colorbar
cbar = plt.colorbar(map, anchor=(0.0,0.0), format="%.0f",#
 location="right", orientation="vertical", aspect=11, ticks=[-139, -92, -47, 0, 47])
for A in cbar.ax.get_yticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_x(2.2)
cbar.ax.tick_params(labelsize=13)
cbar.set_label(r'$E_z\ [\mathrm{\frac{kV}{m}}]$', fontsize=15, rotation=270, labelpad=20)
##
# ax.set_ylim(0,0.5)
# ax.set_xlabel(r't [ns]', labelpad = 7.0)
ax.set_ylabel(r'z [$\mu$m]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{z\textsubscript{1}}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##############################################
# E zone 2#
##############################################
norm = MidpointNormalize(midpoint=0)
ax = fig.add_subplot(pgrid[1,2])
#
map = ax.contourf(time[670:961], grid[54:91]*1e6, field_filtered[54:91,670:961], 400, cmap=plt.cm.bwr, norm=norm, extend='both')
ax.contour(time[670:961], grid[54:91]*1e6, field_filtered[54:91,670:961], 7, colors="k", linewidths=0.2, linestyles='dashed')
## create colorbar
cbar = plt.colorbar(map, anchor=(0.0,0.0), format="%.0f",#
 location="right", orientation="vertical", aspect=11, ticks=[-34, -17, 0, 17])
for A in cbar.ax.get_yticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_x(1.9)
cbar.ax.tick_params(labelsize=13)
cbar.set_label(r'$E_z\ [\mathrm{\frac{kV}{m}}]$', fontsize=15, rotation=270, labelpad=23)
##
# ax.set_ylim(0,0.5)
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
ax.set_ylabel(r'z [$\mu$m]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{z\textsubscript{2}}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##############################################
plt.savefig("fig_11.jpg",format="JPEG",bbox_inches='tight',dpi=300)
# plt.show()
plt.close()
