#
#   This script serves to generate fig. 13 from paper doi.org/10.1088/1361-6595/adc7d8
#   Generated by Maximilian Klich
#   last tested 8.04.2025 on python version 3.13.2
#

import os
import matplotlib
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
import matplotlib.patches as patches
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import Normalize
from matplotlib.patches import Patch
from matplotlib.lines import Line2D
from PIL import Image
from matplotlib.offsetbox import AnchoredText
from matplotlib.patches import Polygon
from scipy.ndimage import gaussian_filter

plt.style.use('bmh')
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True
plt.rcParams['axes.grid.axis'] = 'both'
plt.rcParams['axes.grid.which'] = 'major'
#
plt.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})
matplotlib.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})

class MidpointNormalize(Normalize):
    def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
        self.midpoint = midpoint
        Normalize.__init__(self, vmin, vmax, clip)

    def __call__(self, value, clip=None):
        # I'm ignoring masked values and all kinds of edge cases to make a
        # simple example...
        x, y = [self.vmin, self.midpoint, self.vmax], [0, 0.5, 1]
        return np.ma.masked_array(np.interp(value, x, y))

##############################################
print("loading ionization data ...")
N2_dir = np.loadtxt('data/ionization_rate_N2_XT.txt').T
N2_Pen = np.loadtxt('data/ionization_rate_N2_Penning_XT.txt').T
grid = np.loadtxt('data/gridcells.txt', skiprows=1)
time = np.loadtxt('data/time.txt', skiprows=1)

#gain control over the notation
order = np.floor(np.log10(np.abs(np.amax(N2_Pen)))).astype(int)

time = time/time[-1]

ion1 = N2_dir[0:751,:]*10.0**(-order)
ion2 = N2_Pen[0:751,:]*10.0**(-order)

#filter the data to improve the statistics
ion1_filtered = gaussian_filter(ion1, 1.3, order=0)
ion2_filtered = gaussian_filter(ion2, 1.3, order=0)

##############################################
print("plotting ionization data ...")
fig = plt.figure(figsize=(9.1,4.5))
pgrid = plt.GridSpec(1, 2, hspace=0.34, wspace=0.34)
plt.gcf().subplots_adjust(bottom=0.17, left=0.1, right=0.91, top = 0.95)
#
ax = fig.add_subplot(pgrid[0,0])
#
map = ax.contourf(time, grid[0:751]*1e3, ion1_filtered, 400, cmap=plt.cm.turbo, extend='both')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.2f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('bottom')
    A.set_horizontalalignment('center')
    A.set_y(0.9)
    A.set_rotation(70)
## add exponent
ax.text(.95, 1.19, r'$\times 10^{{{expo}}}$'.format(expo=order), transform=ax.transAxes, c='k')
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{a)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## mark the zoomed zones
rect1 = patches.Rectangle((time[35],0), time[331]-time[35], grid[51]*1e3, linewidth=2, edgecolor='w', facecolor='none')
ax.add_patch(rect1)
ax.text(time[120], grid[55]*1e3, r'\textbf{z\textsubscript{1}}', fontsize=13, color='w', ha='left', va='bottom')
rect2 = patches.Rectangle((time[670], grid[54]*1e3), time[961]-time[670], (grid[91]-grid[54])*1e3, linewidth=2, edgecolor='w', facecolor='none')
ax.add_patch(rect2)
ax.text(time[740], grid[95]*1e3, r'\textbf{z\textsubscript{2}}', fontsize=13, color='w', ha='left', va='bottom')
## add mode label
at = AnchoredText(r'\textbf{III}', frameon=True, loc='upper left',#
 bbox_to_anchor=(-0.27,1.31), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:green')
at.patch.set_alpha(0.4)
ax.add_artist(at)
##############################################
# Penning #
##############################################
ax = fig.add_subplot(pgrid[0,1])
#
map = ax.contourf(time, grid[0:751]*1e3, ion2_filtered, 400, cmap=plt.cm.turbo, extend='both')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.2f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('bottom')
    A.set_horizontalalignment('center')
    A.set_y(0.9)
    A.set_rotation(70)
## add exponent
ax.text(.95, 1.19, r'$\times 10^{{{expo}}}$'.format(expo=order), transform=ax.transAxes, c='k')
## add row label
at = AnchoredText(r'$S_\mathrm{i,N_2^{\ +},d/P}\ [\mathrm{\frac{m^{-3}}{s}}]$', frameon=True, loc='center left',#
 bbox_to_anchor=(1.07,.5), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17, rotation=270))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{b)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)

##############################################
plt.savefig("fig_13.jpg",format="JPEG",dpi=300)
# plt.show()
