#
#   This script serves to generate fig. 2 from paper doi.org/10.1088/1361-6595/adc7d8
#   Generated by Maximilian Klich
#   last tested 8.04.2025 on python version 3.13.2
#

import os
import matplotlib
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
from matplotlib.patches import Patch
from matplotlib.lines import Line2D
from PIL import Image
from matplotlib.offsetbox import AnchoredText
from matplotlib.patches import Polygon
from scipy.ndimage import gaussian_filter

plt.style.use('bmh')
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True
plt.rcParams['axes.grid.axis'] = 'both'
plt.rcParams['axes.grid.which'] = 'major'
#
plt.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})
matplotlib.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})


##############################################
print("loading density data ...")
data_22 = np.loadtxt('data/ave_powers_220V.txt', skiprows=1).T
data_25 = np.loadtxt('data/ave_powers_250V.txt', skiprows=1).T
data_30 = np.loadtxt('data/ave_powers_300V.txt', skiprows=1).T

data_12 = np.loadtxt('data/ave_powers_12MHz.txt', skiprows=1).T
data_18 = np.loadtxt('data/ave_powers_18MHz.txt', skiprows=1).T
data_34 = np.loadtxt('data/ave_powers_34MHz.txt', skiprows=1).T

##############################################
print("processing density data ...")

order = 17
# normalize the density data to the order of magnitude
data_22[1,:] = data_22[1,:]*10.0**(-order)
data_25[1,:] = data_25[1,:]*10.0**(-order)
data_30[1,:] = data_30[1,:]*10.0**(-order)

# multiply the power densities by the volume of the discharge channel
data_22[2,:] = data_22[2,:]*30e-9
data_25[2,:] = data_25[2,:]*30e-9
data_30[2,:] = data_30[2,:]*30e-9
#
data_12[2,:] = data_12[2,:]*30e-9
data_18[2,:] = data_18[2,:]*30e-9
data_34[2,:] = data_34[2,:]*30e-9

#smooth the data
d22_fil = gaussian_filter(data_22[1,:], 0.7, order=0)
d25_fil = gaussian_filter(data_25[1,:], 0.7, order=0)
d30_fil = gaussian_filter(data_30[1,:], 0.7, order=0)

p22_fil = gaussian_filter(data_22[2,:], 0.7, order=0)
p25_fil = gaussian_filter(data_25[2,:], 0.7, order=0)
p30_fil = gaussian_filter(data_30[2,:], 0.7, order=0)

##############################################
print("plotting density data ...")
fig = plt.figure(figsize=(11,9))
pgrid = plt.GridSpec(2, 2, hspace=0.19, wspace=0.22)
plt.gcf().subplots_adjust(bottom=0.1, left=0.1, right=0.98, top = 0.91)
##############################################
##############################################
ax = fig.add_subplot(pgrid[0,0])
ax.plot(data_22[0,:], d22_fil, c='tab:blue', lw=2, ls='-', marker='o', label=r'$220\,$V')
ax.plot([data_22[0,1], data_22[0,7], data_22[0,21]], [d22_fil[1], d22_fil[7], d22_fil[21]], zorder=4, ls='',#
    c='tab:red', marker='*', ms=12)
ax.plot(data_25[0,:], d25_fil, c='tab:brown', lw=2, ls='-', marker='s', label=r'$250\,$V')
ax.plot(data_30[0,:], d30_fil, c='tab:green', lw=2, ls='-', marker='v', label=r'$300\,$V')
#
ax.set_xlim(9, 51)
ax.set_ylim(-0.5,13.5)
#
# ax.set_xlabel(r'$f_\mathrm{RF}$ [MHz]', labelpad=7.0)
ax.set_ylabel(r'$\bar{{n}}_\mathrm{{e}}$ [$10^{{{expo}}} \mathrm{{\,m}}^{{-3}}$]'.format(expo=order), labelpad=7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
#
ax.legend(loc='lower center', bbox_to_anchor=(0.5,1.03), borderaxespad=0, ncols=3, framealpha=1.0,#
    fontsize=13, facecolor='lightgrey', title=r'\textbf{frequency variation:}')
## add panel label
at = AnchoredText(r'\textbf{a)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
### add the markers for the discharge modes
# non-neutral
polygon_points = [(9,-0.5), (51, -0.5), (51, d22_fil[14]), (25, d22_fil[14]), (18, d30_fil[4]), (18,13.5), (9,13.5)]
polygon = Polygon(polygon_points, closed=True, color='tab:blue', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{I}', frameon=True, loc='center',#
 bbox_to_anchor=(0.1,0.75), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# quasi-neutral
polygon_points = [(18, 13.5), (18, data_30[1,4]), (25, data_22[1, 14]), (51, data_22[1, 14]), (51, 13.5)]
polygon = Polygon(polygon_points, closed=True, color='tab:red', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{II}', frameon=True, loc='center',#
 bbox_to_anchor=(0.4,0.75), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.4)
ax.add_artist(at)
##############################################
##############################################
ax = fig.add_subplot(pgrid[0,1])
ax.semilogy(data_12[0,:], data_12[1,:], c='tab:blue', lw=2, ls='-', marker='o', label=r'$12\,$MHz')
ax.semilogy([data_12[0,2], data_18[0,4], data_34[0,4], data_18[0,14]], [data_12[1,2], data_18[1,4], data_34[1,4], data_18[1,14]],#
    zorder=4, ls='', c='tab:red', marker='*', ms=12)
ax.semilogy(data_18[0,:], data_18[1,:], c='tab:brown', lw=2, ls='-', marker='s', label=r'$18\,$MHz')
ax.semilogy(data_34[0,:], data_34[1,:], c='tab:green', lw=2, ls='-', marker='v', label=r'$34\,$MHz')
#
ax.set_xlim(155, 405)
ax.set_ylim(6e15, 6e19)
# ax.set_xlabel(r'$V_\mathrm{RF}$ [V]', labelpad=7.0)
# ax.set_ylabel(r'$\bar{n}_\mathrm{e}$ [m$^{-3}$]', labelpad=7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
#
ax.legend(loc='lower center', bbox_to_anchor=(0.5,1.03), borderaxespad=0, ncols=3, #
    framealpha=1.0, fontsize=13, facecolor='lightgrey', title=r'\textbf{voltage variation:}')
## add panel label
at = AnchoredText(r'\textbf{b)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
# at.patch.set_boxstyle("Square,pad=0.")
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
### add the markers for the discharge modes
# non-neutral
polygon_points = [(155,6e15), (405, 6e15), (405, 3.2e18), (315, 3.2e18), (315, 1.3e17), (280, 1.3e17), (180, 1e17), (155, 1e17)]
polygon = Polygon(polygon_points, closed=True, color='tab:blue', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{I}', frameon=True, loc='center',#
 bbox_to_anchor=(0.91,0.3), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# quasi-neutral
polygon_points = [(155, 1e17), (180, 1e17), (280, 1.3e17), (315, 1.3e17), (315, 6e19), (155, 6e19)]
polygon = Polygon(polygon_points, closed=True, color='tab:red', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{II}', frameon=True, loc='center',#
 bbox_to_anchor=(0.3,0.75), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# constricted
polygon_points = [(315,6e19), (315, 3.2e18), (405, 3.2e18), (405, 6e19)]
polygon = Polygon(polygon_points, closed=True, color='tab:green', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{III}', frameon=True, loc='center',#
 bbox_to_anchor=(0.91,0.75), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:green')
at.patch.set_alpha(0.4)
ax.add_artist(at)
##############################################
##############################################
ax = fig.add_subplot(pgrid[1,0])
ax.plot(data_22[0,:], p22_fil, c='tab:blue', lw=2, ls='-', marker='o', label=r'$220\,$V')
ax.plot([data_22[0,1], data_22[0,7], data_22[0,21]], [p22_fil[1], p22_fil[7], p22_fil[21]], zorder=4, ls='',#
    c='tab:red', marker='*', ms=12)
ax.plot(data_25[0,:], p25_fil, c='tab:brown', lw=2, ls='-', marker='s', label=r'$250\,$V')
ax.plot(data_30[0,:], p30_fil, c='tab:green', lw=2, ls='-', marker='v', label=r'$300\,$V')
#
#
ax.set_xlim(9, 51)
ax.set_ylim(-0.5,13.5)
#
ax.set_xlabel(r'$f_\mathrm{RF}$ [MHz]', labelpad=7.0)
ax.set_ylabel(r'$P_\mathrm{abs}$ [W]', labelpad=7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
#
## add panel label
at = AnchoredText(r'\textbf{c)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
# at.patch.set_boxstyle("Square,pad=0.")
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
### add the markers for the discharge modes
# non-neutral
polygon_points = [(9,-0.5), (51, -0.5), (51, p22_fil[14]), (25, p22_fil[14]), (18, p30_fil[4]), (18,13.5), (9,13.5)]
polygon = Polygon(polygon_points, closed=True, color='tab:blue', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{I}', frameon=True, loc='center',#
 bbox_to_anchor=(0.1,0.75), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# quasi-neutral
polygon_points = [(18, 13.5), (18, p30_fil[4]), (25, p22_fil[14]), (51, p22_fil[14]), (51, 13.5)]
polygon = Polygon(polygon_points, closed=True, color='tab:red', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{II}', frameon=True, loc='center',#
 bbox_to_anchor=(0.4,0.75), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.4)
ax.add_artist(at)
##############################################
##############################################
ax = fig.add_subplot(pgrid[1,1])
ax.semilogy(data_12[0,:], data_12[2,:], c='tab:blue', lw=2, ls='-', marker='o', label=r'$12\,$MHz')
ax.semilogy([data_12[0,2], data_18[0,4], data_34[0,4], data_18[0,14]], [data_12[2,2], data_18[2,4], data_34[2,4], data_18[2,14]],#
    zorder=4, ls='', c='tab:red', marker='*', ms=12)
ax.semilogy(data_18[0,:], data_18[2,:], c='tab:brown', lw=2, ls='-', marker='s', label=r'$18\,$MHz')
ax.semilogy(data_34[0,:], data_34[2,:], c='tab:green', lw=2, ls='-', marker='v', label=r'$34\,$MHz')
#
ax.set_xlim(155, 405)
ax.set_ylim(5e-2, 1000)
ax.set_xlabel(r'$V_\mathrm{RF}$ [V]', labelpad=7.0)
# ax.set_ylabel(r'$P_\mathrm{abs}$ [W]', labelpad=7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
#
## add panel label
at = AnchoredText(r'\textbf{d)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
# at.patch.set_boxstyle("Square,pad=0.")
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
### add the markers for the discharge modes
# # non-neutral
polygon_points = [(155,5e-2), (405, 5e-2), (405, 40), (315, 40), (315, 1.1), (280, 1.1), (180, 0.65), (155, 0.65)]
polygon = Polygon(polygon_points, closed=True, color='tab:blue', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{I}', frameon=True, loc='center',#
 bbox_to_anchor=(0.91,0.3), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# quasi-neutral
polygon_points = [(155, 0.65), (180, 0.65), (280, 1.1), (315, 1.1), (315,40), (315, 1000), (155, 1000)]
polygon = Polygon(polygon_points, closed=True, color='tab:red', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{II}', frameon=True, loc='center',#
 bbox_to_anchor=(0.3,0.75), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# constricted
polygon_points = [(315,1000), (315, 40), (405, 40), (405, 1000)]
polygon = Polygon(polygon_points, closed=True, color='tab:green', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{III}', frameon=True, loc='center',#
 bbox_to_anchor=(0.91,0.75), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:green')
at.patch.set_alpha(0.4)
ax.add_artist(at)
######################
##############################################
plt.savefig("fig_2.jpg",format="JPEG",dpi=300)
# plt.show()
