#
#   This script serves to generate fig. 3 from paper doi.org/10.1088/1361-6595/adc7d8
#   Generated by Maximilian Klich
#   last tested 8.04.2025 on python version 3.13.2
#

import os
import matplotlib
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
from matplotlib.patches import Patch
from matplotlib.lines import Line2D
from PIL import Image
from matplotlib.offsetbox import AnchoredText
from matplotlib.patches import Polygon
from scipy.ndimage import gaussian_filter

plt.style.use('bmh')
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True
plt.rcParams['axes.grid.axis'] = 'both'
plt.rcParams['axes.grid.which'] = 'major'
#
plt.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})
matplotlib.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})


##############################################
print("loading density data ...")
data_22 = np.loadtxt('data/sheath_width_220V.txt', skiprows=1).T
data_25 = np.loadtxt('data/sheath_width_250V.txt', skiprows=1).T
data_30 = np.loadtxt('data/sheath_width_300V.txt', skiprows=1).T

phi_22 = np.loadtxt('data/phase_shifts_220V.txt', skiprows=1).T
phi_25 = np.loadtxt('data/phase_shifts_250V.txt', skiprows=1).T
phi_30 = np.loadtxt('data/phase_shifts_300V.txt', skiprows=1).T

data_12 = np.loadtxt('data/sheath_width_12MHz.txt', skiprows=1).T
data_18 = np.loadtxt('data/sheath_width_18MHz.txt', skiprows=1).T
data_34 = np.loadtxt('data/sheath_width_34MHz.txt', skiprows=1).T

phi_12 = np.loadtxt('data/phase_shifts_12MHz.txt', skiprows=1).T
phi_18 = np.loadtxt('data/phase_shifts_18MHz.txt', skiprows=1).T
phi_34 = np.loadtxt('data/phase_shifts_34MHz.txt', skiprows=1).T

##############################################
print("processing density data ...")

# transform the length scales to mm
data_22[1,:] = data_22[1,:]*1e3
data_25[1,:] = data_25[1,:]*1e3
data_30[1,:] = data_30[1,:]*1e3

data_12[1,:] = data_12[1,:]*1e3
data_18[1,:] = data_18[1,:]*1e3
data_34[1,:] = data_34[1,:]*1e3

#smooth the data
s22_fil = gaussian_filter(data_22[1,:], 0.7, order=0)
s25_fil = gaussian_filter(data_25[1,:], 0.7, order=0)
s30_fil = gaussian_filter(data_30[1,:], 0.7, order=0)

p22_fil = gaussian_filter(phi_22[1,:], 0.7, order=0)
p25_fil = gaussian_filter(phi_25[1,:], 0.7, order=0)
p30_fil = gaussian_filter(phi_30[1,:], 0.7, order=0)

##############################################
print("plotting density data ...")
fig = plt.figure(figsize=(11,9))
pgrid = plt.GridSpec(2, 2, hspace=0.22, wspace=0.18)
plt.gcf().subplots_adjust(bottom=0.1, left=0.08, right=0.98, top = 0.91)
##############################################
##############################################
ax = fig.add_subplot(pgrid[0,0])
ax.plot(data_22[0,:], s22_fil, c='tab:blue', lw=2, ls='-', marker='o', label=r'$220\,$V', zorder=4)
ax.plot([data_22[0,1], data_22[0,7], data_22[0,21]], [s22_fil[1], s22_fil[7], s22_fil[21]], zorder=4, ls='',#
    c='tab:red', marker='*', ms=12)
ax.plot(data_25[0,:], s25_fil, c='tab:brown', lw=2, ls='-', marker='s', label=r'$250\,$V', zorder=3)
ax.plot(data_30[0,:], s30_fil, c='tab:green', lw=2, ls='-', marker='v', label=r'$300\,$V', zorder=3)
#
ax.plot([9,51], [1.0, 1.0], c='k', lw=1, ls='--', zorder=2)
ax.plot([9,51], [0.5, 0.5], c='k', lw=1, ls='-.', zorder=2)
#
ax.set_xlim(9, 51)
ax.set_ylim(0.0, 2.0)
#
# ax.set_xlabel(r'$f_\mathrm{RF}$ [MHz]', labelpad=7.0)
ax.set_ylabel(r'$\bar{s}$ [mm]', labelpad=7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
#
ax.legend(loc='lower center', bbox_to_anchor=(.5,1.03), borderaxespad=0, framealpha=1.0, ncols=3,#
    fontsize=13, handlelength = 1.8, handletextpad = 0.7, facecolor='lightgrey',#
    title = r"\textbf{frequency variation:}")
## add panel label
at = AnchoredText(r'\textbf{a)}', frameon=True, loc='upper right',#
 bbox_to_anchor=(0.99,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## add the markers for the discharge modes
# non-neutral
polygon_points = [(9,s30_fil[4]), (18, s30_fil[4]), (25, s22_fil[14]), (51, s22_fil[14]), (51,2.3), (9,2.3)]
polygon = Polygon(polygon_points, closed=True, color='tab:blue', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{I}', frameon=True, loc='center',#
 bbox_to_anchor=(0.56,0.42), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# quasi-neutral
polygon_points = [(9,0), (51,0), (51, s22_fil[14]), (25, s22_fil[14]), (18, s30_fil[4]), (0, s30_fil[4])]
polygon = Polygon(polygon_points, closed=True, color='tab:red', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{II}', frameon=True, loc='center',#
 bbox_to_anchor=(0.15,0.08), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.4)
ax.add_artist(at)
##############################################
##############################################
ax = fig.add_subplot(pgrid[0,1])
ax.plot(data_12[0,:], data_12[1,:], c='tab:blue', lw=2, ls='-', marker='o', label=r'$12\,$MHz', zorder=4)   #cyan
ax.plot([data_12[0,2], data_18[0,4], data_34[0,4], data_18[0,14]], [data_12[1,2], data_18[1,4], data_34[1,4], data_18[1,14]],#
    zorder=4, ls='', c='tab:red', marker='*', ms=12)
ax.plot(data_18[0,:], data_18[1,:], c='tab:brown', lw=2, ls='-', marker='s', label=r'$18\,$MHz', zorder=3)    #red
ax.plot(data_34[0,:], data_34[1,:], c='tab:green', lw=2, ls='-', marker='v', label=r'$34\,$MHz', zorder=3) #purple
#
ax.plot([155,405], [1.0, 1.0], c='k', lw=1, ls='--', zorder=2)
ax.plot([155,405], [0.5, 0.5], c='k', lw=1, ls='-.', zorder=2)
#
ax.set_xlim(155, 405)
ax.set_ylim(0, 1.55)
# ax.set_xlabel(r'$V_\mathrm{RF}$ [V]', labelpad=7.0)
# ax.set_ylabel(r'$\bar{n}_\mathrm{e}$ [m$^{-3}$]', labelpad=7.0)
ax.set_yticks([0, 0.5, 1.0, 1.5])
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
#
ax.legend(loc='lower center', bbox_to_anchor=(.5,1.03), borderaxespad=0, framealpha=1.0, ncols=3,#
    fontsize=13, handlelength = 1.8, handletextpad = 0.7, facecolor='lightgrey',#
    title=r"\textbf{voltage variation:}")
## add panel label
at = AnchoredText(r'\textbf{b)}', frameon=True, loc='upper right',#
 bbox_to_anchor=(.99,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
### add the markers for the discharge modes
# non-neutral
polygon_points = [(155, 1.55), (155,0.5), (315, 0.5), (315,0.15), (405,0.15), (405, 1.55)]
polygon = Polygon(polygon_points, closed=True, color='tab:blue', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{I}', frameon=True, loc='center',#
 bbox_to_anchor=(0.91,0.42), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# quasi-neutral
polygon_points = [(155, 0), (315, 0), (315, 0.15), (315, 0.5), (180, 0.5), (155, 0.5)]
polygon = Polygon(polygon_points, closed=True, color='tab:red', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{II}', frameon=True, loc='center',#
 bbox_to_anchor=(0.26,0.08), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# constricted
polygon_points = [(315,0), (315, 0.15), (405, 0.15), (405, 0)]
polygon = Polygon(polygon_points, closed=True, color='tab:green', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{III}', frameon=True, loc='center',#
 bbox_to_anchor=(0.91,0.08), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12), zorder=2)
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:green')
at.patch.set_alpha(0.4)
ax.add_artist(at)
##############################################
##############################################
ax = fig.add_subplot(pgrid[1,0])
ax.plot(phi_22[0,:], p22_fil, c='tab:blue', lw=2, ls='-', marker='o', label=r'$220\,$V', zorder=3)
ax.plot([phi_22[0,1], phi_22[0,7], phi_22[0,21]], [p22_fil[1], p22_fil[7], p22_fil[21]], zorder=4, ls='',#
    c='tab:red', marker='*', ms=12)
ax.plot(phi_25[0,:], p25_fil, c='tab:brown', lw=2, ls='-', marker='s', label=r'$250\,$V', zorder=2)
ax.plot(phi_30[0,:], p30_fil, c='tab:green', lw=2, ls='-', marker='v', label=r'$300\,$V', zorder=2)
#
ax.set_xlim(9, 51)
ax.set_ylim(47, 82)
#
ax.set_xlabel(r'$f_\mathrm{RF}$ [MHz]', labelpad=7.0)
ax.set_ylabel(r'$\Delta{\varphi}$ [$\ ^\circ$]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{c)}', frameon=True, loc='upper right',#
 bbox_to_anchor=(.99,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## add the markers for the discharge modes
# non-neutral
polygon_points = [(9,p30_fil[4]), (18, p30_fil[4]), (25, p22_fil[14]), (51, p22_fil[14]), (51,85), (9,85)]
polygon = Polygon(polygon_points, closed=True, color='tab:blue', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{I}', frameon=True, loc='center',#
 bbox_to_anchor=(0.56,0.67), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# quasi-neutral
polygon_points = [(9,0), (51,0), (51, p22_fil[14]), (25, p22_fil[14]), (18, p30_fil[4]), (9,p30_fil[4])]
polygon = Polygon(polygon_points, closed=True, color='tab:red', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{II}', frameon=True, loc='center',#
 bbox_to_anchor=(0.15,0.19), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.4)
ax.add_artist(at)
##############################################
##############################################
ax = fig.add_subplot(pgrid[1,1])
ax.plot(phi_12[0,:], phi_12[1,:], c='tab:blue', lw=2, ls='-', marker='o', label=r'$12\,$MHz', zorder=2)
ax.plot([phi_12[0,2], phi_18[0,4], phi_34[0,4], phi_18[0,14]], [phi_12[1,2], phi_18[1,4], phi_34[1,4], phi_18[1,14]],#
    zorder=4, ls='', c='tab:red', marker='*', ms=12)
ax.plot(phi_18[0,:], phi_18[1,:], c='tab:brown', lw=2, ls='-', marker='s', label=r'$18\,$MHz', zorder=2)
ax.plot(phi_34[0,:], phi_34[1,:], c='tab:green', lw=2, ls='-', marker='v', label=r'$34\,$MHz', zorder=2)
#
ax.set_xlim(155,405)
ax.set_ylim(4, 83)
ax.set_xlabel(r'$V_\mathrm{RF}$ [V]', labelpad=7.0)
# ax.set_ylabel(r'$P_\mathrm{abs}$ [W]', labelpad=7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{d)}', frameon=True, loc='upper right',#
 bbox_to_anchor=(.99,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
### add the markers for the discharge modes
# non-neutral
polygon_points = [(155, 83), (155,63), (180, 63), (315, 64), (315,25), (405,25), (405, 83)]
polygon = Polygon(polygon_points, closed=True, color='tab:blue', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{I}', frameon=True, loc='center',#
 bbox_to_anchor=(0.91,0.67), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# quasi-neutral
polygon_points = [(155, 0), (315, 0), (315, 25), (315, 64), (180, 63), (155, 63)]
polygon = Polygon(polygon_points, closed=True, color='tab:red', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{II}', frameon=True, loc='center',#
 bbox_to_anchor=(0.26,0.19), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.4)
ax.add_artist(at)
# constricted
polygon_points = [(315,0), (315, 25), (405, 25), (405, 0)]
polygon = Polygon(polygon_points, closed=True, color='tab:green', alpha=0.3)
ax.add_patch(polygon)
at = AnchoredText(r'\textbf{III}', frameon=True, loc='center',#
 bbox_to_anchor=(0.91,0.19), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=12), zorder=2)
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:green')
at.patch.set_alpha(0.4)
ax.add_artist(at)
######################
##############################################
plt.savefig("fig_3.jpg",format="JPEG",dpi=300)
# plt.show()
