#
#   This script serves to generate fig. 6 from paper doi.org/10.1088/1361-6595/adc7d8
#   Generated by Maximilian Klich
#   last tested 8.04.2025 on python version 3.13.2
#

import os
import matplotlib
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import Normalize
from matplotlib.patches import Patch
from matplotlib.lines import Line2D
from PIL import Image
from matplotlib.offsetbox import AnchoredText
from matplotlib.patches import Polygon
from scipy.ndimage import gaussian_filter

plt.style.use('bmh')
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True
plt.rcParams['axes.grid.axis'] = 'both'
plt.rcParams['axes.grid.which'] = 'major'
#
plt.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})
matplotlib.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})

class MidpointNormalize(Normalize):
    def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
        self.midpoint = midpoint
        Normalize.__init__(self, vmin, vmax, clip)

    def __call__(self, value, clip=None):
        # I'm ignoring masked values and all kinds of edge cases to make a
        # simple example...
        x, y = [self.vmin, self.midpoint, self.vmax], [0, 0.5, 1]
        return np.ma.masked_array(np.interp(value, x, y))

##############################################
print("loading density data ...")
simu = np.loadtxt('data/exc_rate_He33S_XT.txt').T
expe = np.loadtxt('data/A4set00611_exc_matrix_cut_IK2.dat')
grid = np.loadtxt('data/gridcells.txt', skiprows=1)
time = np.loadtxt('data/time.txt', skiprows=1)

time /= time[-1]

order = np.floor(np.log10(np.abs(np.amax(simu)))).astype(int)

ngp = expe.shape[0]
nts = expe.shape[1]

grid_exp = np.linspace(0,1, num=ngp, endpoint=True)
time_exp = np.linspace(0,1, num=nts, endpoint=True)

simu_filtered = gaussian_filter(simu*10.0**(-order), 1.3, order=0)

##############################################
print("plotting density data ...")
fig = plt.figure(figsize=(9.0,4.5))
pgrid = plt.GridSpec(1, 2, hspace=0.34, wspace=0.34)
plt.gcf().subplots_adjust(bottom=0.16, left=0.1, right=0.91, top = 0.96)
##############################################
##############################################
ax = fig.add_subplot(pgrid[0,0])
norm = MidpointNormalize(midpoint=0)
map = ax.contourf(time, grid*1e3, simu_filtered, 400, cmap=plt.cm.turbo, extend='both')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_y(2.3)
    A.set_rotation(70)
## add exponent
ax.text(.95, 1.17, r'$\times 10^{{{expo}}}$'.format(expo=order), transform=ax.transAxes, c='k')
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
ax.set_ylabel(r'z [mm]', labelpad = 8.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{a)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{III}', frameon=True, loc='upper left',#
 bbox_to_anchor=(-0.27,1.29), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:green')
at.patch.set_alpha(0.4)
ax.add_artist(at)
##############################################
# He_706nm experiment #
##############################################
ax = fig.add_subplot(pgrid[0,1])
#
map = ax.contourf(time_exp, grid_exp, expe, 400, cmap=plt.cm.turbo, extend='both')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.0f",#
 location="top", orientation="horizontal", shrink=0.94, ticks=[0, 75, 150, 225, 300, 375, 450, 525, 600, 675])
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_y(2.3)
    A.set_rotation(70)
## add row label
at = AnchoredText(r'$R_\mathrm{He(3s)\ ^3S_1,sim/exp}\ [\mathrm{\frac{m^{-3}}{s}} / a.u.]$', frameon=True, loc='center left',#
 bbox_to_anchor=(1.07,.5), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17, rotation=270))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{b)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##############################################
##############################################
plt.savefig("fig_6.jpg",format="JPEG",dpi=300)
# plt.show()
