#
#   This script serves to generate fig. 7 from paper doi.org/10.1088/1361-6595/adc7d8
#   Generated by Maximilian Klich
#   last tested 8.04.2025 on python version 3.13.2
#

import os
import matplotlib
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
from matplotlib.patches import Patch
from matplotlib.lines import Line2D
from PIL import Image
from matplotlib.offsetbox import AnchoredText
from matplotlib.patches import Polygon
from scipy.ndimage import gaussian_filter

plt.style.use('bmh')
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True
plt.rcParams['axes.grid.axis'] = 'both'
plt.rcParams['axes.grid.which'] = 'major'
#
plt.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})
matplotlib.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})

##############################################
print("loading density data ...")
#data 12 MHz #
ne_12 = np.loadtxt('data/12MHz/ne_XT.txt').T
nHe_12 = np.loadtxt('data/12MHz/ni_He+_XT.txt').T
nH2_12 = np.loadtxt('data/12MHz/ni_He2+_XT.txt').T
nN2_12 = np.loadtxt('data/12MHz/ni_N2+_XT.txt').T
nN4_12 = np.loadtxt('data/12MHz/ni_N4+_XT.txt').T
grid_12 = np.loadtxt('data/12MHz/gridcells.txt', skiprows=1)
time_12 = np.loadtxt('data/12MHz/time.txt', skiprows=1)
# data 18 MHz #
ne_18 = np.loadtxt('data/18MHz/ne_XT.txt').T
nHe_18 = np.loadtxt('data/18MHz/ni_He+_XT.txt').T
nH2_18 = np.loadtxt('data/18MHz/ni_He2+_XT.txt').T
nN2_18 = np.loadtxt('data/18MHz/ni_N2+_XT.txt').T
nN4_18 = np.loadtxt('data/18MHz/ni_N4+_XT.txt').T
grid_18 = np.loadtxt('data/18MHz/gridcells.txt', skiprows=1)
time_18 = np.loadtxt('data/18MHz/time.txt', skiprows=1)
#data 34 MHz #
ne_34 = np.loadtxt('data/34MHz/ne_XT.txt').T
nHe_34 = np.loadtxt('data/34MHz/ni_He+_XT.txt').T
nH2_34 = np.loadtxt('data/34MHz/ni_He2+_XT.txt').T
nN2_34 = np.loadtxt('data/34MHz/ni_N2+_XT.txt').T
nN4_34 = np.loadtxt('data/34MHz/ni_N4+_XT.txt').T
grid_34 = np.loadtxt('data/34MHz/gridcells.txt', skiprows=1)
time_34 = np.loadtxt('data/34MHz/time.txt', skiprows=1)

print("processing density data ...")
ni_12 = nHe_12 + nH2_12 + nN2_12 + nN4_12
ni_18 = nHe_18 + nH2_18 + nN2_18 + nN4_18
ni_34 = nHe_34 + nH2_34 + nN2_34 + nN4_34

#average the ion data - time-dependency is negligible
ni_ave_12 = np.trapezoid(ni_12, x=time_12, dx=time_12[1], axis=1) / time_12[-1]
ni_ave_18 = np.trapezoid(ni_18, x=time_18, dx=time_18[1], axis=1) / time_18[-1]
ni_ave_34 = np.trapezoid(ni_34, x=time_34, dx=time_34[1], axis=1) / time_34[-1]

#normalize the time to the periode length
time_12 /= time_12[-1]
time_18 /= time_18[-1]
time_34 /= time_34[-1]

#normalize rho to gain control over the notation
order = np.floor(np.log10(np.abs(np.amax(ni_ave_18)))).astype(int)
ni_max12 = 0.5
ni_max18 = 1.25
ni_max34 = 4.0

##############################################
print("plotting density data ...")
fig = plt.figure(figsize=(12.5,4.5))
pgrid = plt.GridSpec(1, 3, hspace=0.32, wspace=0.32)
plt.gcf().subplots_adjust(bottom=0.16, left=0.07, right=0.93, top = 0.88)
##############################################
# 12 MHz #
##############################################
ax = fig.add_subplot(pgrid[0,0])
ax.plot(grid_12*1e3, ni_ave_12*10.0**(-order), c='tab:red', lw=2, label=r"$\bar{n_\mathrm{i}}$")
ax.plot(grid_12*1e3, ne_12[:,650]*10.0**(-order), c='tab:blue', lw=2, label=r"$n_\mathrm{e}(t=t_1)$")
ax.plot(grid_12*1e3, ne_12[:,850]*10.0**(-order), c='tab:blue', lw=2, ls='--', label=r"$n_\mathrm{e}(t=t_2)$")
#
ax.annotate('', xy=(0.9, 0.2), xytext=(0.69, 0.2), arrowprops=dict(facecolor='black', shrink=0.05))
ax.text(0.33, 0.13, r'$t_{1,a}$', c='k')
ax.text(0.87, 0.33, r'$t_{2,a}$', c='k')
#
ax.set_xlim(0.0, 1.0)
ax.set_ylim(0.0, ni_max12)
#
ax.set_xlabel(r'z [mm]', labelpad=7.0)
ax.set_ylabel(r'$n_s$ [$10^{{{expo}}} \mathrm{{\,m}}^{{-3}}$]'.format(expo=order), labelpad=7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
#
## add panel label
at = AnchoredText(r'\textbf{I}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.4)
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{non-neutral mode}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.03), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.5)
ax.add_artist(at)
### add the markers for the discharge modes
polygon_points = [(0,0), (0,ni_max12), (1,ni_max12), (1,0)]
polygon = Polygon(polygon_points, closed=True, color='tab:blue', alpha=0.3)
ax.add_patch(polygon)
##############################################
# 18 MHz #
##############################################
ax = fig.add_subplot(pgrid[0,1])
ax.plot(grid_18*1e3, ni_ave_18*10.0**(-order), c='tab:red', lw=2, label=r"$\bar{n_\mathrm{i}}$")
ax.plot(grid_18*1e3, ne_18[:,480]*10.0**(-order), c='tab:blue', lw=2, label=r"$n_\mathrm{e}(t=t_1)$")
ax.plot(grid_18*1e3, ne_18[:,800]*10.0**(-order), c='tab:blue', lw=2, ls='--', label=r"$n_\mathrm{e}(t=t_2)$")
#
ax.annotate('', xy=(0.88, 0.5), xytext=(0.66, 0.5), arrowprops=dict(facecolor='black', shrink=0.05))
ax.text(0.1, 0.3, r'$t_{1,b}$', c='k')
ax.text(0.82, 0.91, r'$t_{2,b}$', c='k')
#
ax.set_xlim(0.0, 1.0)
ax.set_ylim(0.0, ni_max18)
#
ax.set_xlabel(r'z [mm]', labelpad=7.0)
# ax.set_ylabel(r'$\bar{{n}}_s$ [$10^{{{expo}}} \mathrm{{\,m}}^{{-3}}$]'.format(expo=order_qne), labelpad=7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
#
## add panel label
at = AnchoredText(r'\textbf{I/II}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:purple')
at.patch.set_alpha(0.4)
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{transition}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.03), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:purple')
at.patch.set_alpha(0.5)
ax.add_artist(at)
### add the markers for the discharge modes
polygon_points = [(0,0), (0,ni_max18), (1,ni_max18), (1,0)]
polygon = Polygon(polygon_points, closed=True, color='tab:purple', alpha=0.3)
ax.add_patch(polygon)
##############################################
# 34 MHz #
##############################################
ax = fig.add_subplot(pgrid[0,2])
ax.plot(grid_34*1e3, ni_ave_34*10.0**(-order), c='tab:red', lw=2, label=r"$\bar{n_\mathrm{i}}$")
ax.plot(grid_34*1e3, ne_34[:,450]*10.0**(-order), c='tab:blue', lw=2, label=r"$n_\mathrm{e}(t_{1,\ast})$")
ax.plot(grid_34*1e3, ne_34[:,750]*10.0**(-order), c='tab:blue', lw=2, ls='--', label=r"$n_\mathrm{e}(t_{2,\ast})$")
#
ax.annotate('', xy=(0.91, 1.6), xytext=(0.76, 1.6), arrowprops=dict(facecolor='black', shrink=0.05))
ax.text(0.05, 0.7, r'$t_{1,c}$', c='k')
ax.text(0.82, 3.1, r'$t_{2,c}$', c='k')
#
ax.set_xlim(0.0, 1.0)
ax.set_ylim(0.0, ni_max34)
#
ax.set_xlabel(r'z [mm]', labelpad=7.0)
# ax.set_ylabel(r'$\bar{{n}}_s$ [$10^{{{expo}}} \mathrm{{\,m}}^{{-3}}$]'.format(expo=order_con), labelpad=7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
#
## add panel label
at = AnchoredText(r'\textbf{II}', frameon=True, loc='upper left',#
 bbox_to_anchor=(0.01,0.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16), zorder=2)
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.4)
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{quasi-neutral mode}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.03), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.5)
ax.add_artist(at)
### add the markers for the discharge modes
polygon_points = [(0,0), (0,ni_max34), (1,ni_max34), (1,0)]
polygon = Polygon(polygon_points, closed=True, color='tab:red', alpha=0.3)
ax.add_patch(polygon)
####
#### manually add the axis for the entire plot
####
# get handles and labels
handles, labels = ax.get_legend_handles_labels()
# create a new axis
leg_ax = fig.add_axes([0.95, 0.15, 0.04, 0.74], frameon=False)
leg_ax.set_xlim(0, 1)
leg_ax.set_ylim(0, 1)
leg_ax.axis('off')
#manually draw the legend
leg_ax.plot([0.5, 0.5], [0.95, 0.88], marker='none', color='tab:red', ls='-', lw=2)
leg_ax.text(0.37, 0.82, labels[0], verticalalignment='center', rotation=270)
leg_ax.plot([0.5, 0.5], [0.73, 0.66], marker='none', color='tab:blue', ls='-', lw=2)
leg_ax.text(0.37, 0.52, labels[1], verticalalignment='center', rotation=270)
leg_ax.plot([0.5, 0.5], [0.35, 0.27], marker='none', color='tab:blue', ls='--', lw=2)
leg_ax.text(0.37, 0.135, labels[2], verticalalignment='center', rotation=270)
# draw the box around the legend
from matplotlib.patches import FancyBboxPatch
bbox = FancyBboxPatch((0, 0), 1, 1, boxstyle="round, pad=0., rounding_size=0.15", 
    edgecolor='none', facecolor='lightgrey')
leg_ax.add_patch(bbox)
##############################################
##############################################
plt.savefig("fig_7.jpg",format="JPEG",dpi=300)
# plt.show()
