#
#   This script serves to generate fig. 9 from paper doi.org/10.1088/1361-6595/adc7d8
#   Generated by Maximilian Klich
#   last tested 8.04.2025 on python version 3.13.2
#

import os
import matplotlib
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import Normalize
from matplotlib.patches import Patch
from matplotlib.lines import Line2D
from PIL import Image
from matplotlib.offsetbox import AnchoredText
from matplotlib.patches import Polygon
from scipy.ndimage import gaussian_filter

plt.style.use('bmh')
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True
plt.rcParams['axes.grid.axis'] = 'both'
plt.rcParams['axes.grid.which'] = 'major'
#
plt.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})
matplotlib.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.size": 16, "text.latex.preamble": r'\usepackage[cm]{sfmath} \usepackage{bm}'})

class MidpointNormalize(Normalize):
    def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
        self.midpoint = midpoint
        Normalize.__init__(self, vmin, vmax, clip)

    def __call__(self, value, clip=None):
        # I'm ignoring masked values and all kinds of edge cases to make a
        # simple example...
        x, y = [self.vmin, self.midpoint, self.vmax], [0, 0.5, 1]
        return np.ma.masked_array(np.interp(value, x, y))

##############################################
# data taken from 220V frequency variation

print("loading data ...")
#data 12 MHz #
je_12 = np.loadtxt('data/12MHz/je_XT.txt').T
Ext_12 = np.loadtxt('data/12MHz/E_XT.txt').T
Te_12 = np.loadtxt('data/12MHz/Te_XT.txt')
grid_12 = np.loadtxt('data/12MHz/gridcells.txt', skiprows=1)
time_12 = np.loadtxt('data/12MHz/time.txt', skiprows=1)
# data 18 MHz #
je_18 = np.loadtxt('data/18MHz/je_XT.txt').T
Ext_18 = np.loadtxt('data/18MHz/E_XT.txt').T
Te_18 = np.loadtxt('data/18MHz/Te_XT.txt')
grid_18 = np.loadtxt('data/18MHz/gridcells.txt', skiprows=1)
time_18 = np.loadtxt('data/18MHz/time.txt', skiprows=1)
#data 34 MHz #
je_34 = np.loadtxt('data/34MHz/je_XT.txt').T
Ext_34 = np.loadtxt('data/34MHz/E_XT.txt').T
Te_34 = np.loadtxt('data/34MHz/Te_XT.txt')
grid_34 = np.loadtxt('data/34MHz/gridcells.txt', skiprows=1)
time_34 = np.loadtxt('data/34MHz/time.txt', skiprows=1)

print("processing data ...")
#normalize the time to the periode length
time_12 /= time_12[-1]
time_18 /= time_18[-1]
time_34 /= time_34[-1]

#calculate an normalize the power density
wxt_12 = je_12 * Ext_12 * 1e-6
wxt_18 = je_18 * Ext_18 * 1e-6
wxt_34 = je_34 * Ext_34 * 1e-6

#cap the values of Te at 6eV
nts = Te_12.shape[1]
ngp = Te_12.shape[0]

Te_max = 6.0
for t in range(0,nts):
    for x in range(0,ngp):
        if(Te_12[x,t] > Te_max):
            Te_12[x,t] = Te_max
        if(Te_18[x,t] > Te_max):
            Te_18[x,t] = Te_max
        if(Te_34[x,t] > Te_max):
            Te_34[x,t] = Te_max

#filter the data to improve the statistics
wxt_12_fil = gaussian_filter(wxt_12, .9, order=0)
Te_12_fil = gaussian_filter(Te_12, .9, order=0)
wxt_18_fil = gaussian_filter(wxt_18, .9, order=0)
Te_18_fil = gaussian_filter(Te_18, .9, order=0)
wxt_34_fil = gaussian_filter(wxt_34, .9, order=0)
Te_34_fil = gaussian_filter(Te_34, .9, order=0)

##############################################
print("plotting electron data ...")
fig = plt.figure(figsize=(13.5,8))
pgrid = plt.GridSpec(2, 3, hspace=0.28, wspace=0.3)
plt.gcf().subplots_adjust(bottom=0.16, left=0.1, right=0.98, top = 0.94)
#
##############################################
# We_12
##############################################
ax = fig.add_subplot(pgrid[0,0])
norm = MidpointNormalize(midpoint=0)
map = ax.contourf(time_12, grid_12*1e3, wxt_12_fil, 400, cmap=plt.cm.bwr, norm=norm, extend='both')
## mark the times for the snapshots ##
ax.plot([time_12[650], time_12[650]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_12[650], time_12[650]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.67, 0.16, r'$t_{1,a}$', c='k')
ax.plot([time_12[850], time_12[850]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_12[850], time_12[850]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.88, 0.16, r'$t_{2,a}$', c='k')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.0f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_y(2.3)
    A.set_rotation(70)
##
# ax.set_xlabel(r't [ns]', labelpad = 7.0)
ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{a)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{non-neutral mode}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.4), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:blue')
at.patch.set_alpha(0.5)
ax.add_artist(at)
##############################################
# Te_12 #
##############################################
ax = fig.add_subplot(pgrid[1,0])
#
map = ax.contourf(time_12, grid_12*1e3, Te_12_fil, 400, cmap=plt.cm.turbo, extend='both')
ax.contour(time_12, grid_12*1e3, Te_12_fil, 7, colors="w", linewidths=0.2, linestyles='dashed')
## mark the times for the snapshots ##
ax.plot([time_12[650], time_12[650]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_12[650], time_12[650]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.67, 0.16, r'$t_{1,a}$', c='k')
ax.plot([time_12[850], time_12[850]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_12[850], time_12[850]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.88, 0.16, r'$t_{2,a}$', c='k')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_y(2.3)
    A.set_rotation(70)
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{b)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
#
##############################################
# We_18
##############################################
ax = fig.add_subplot(pgrid[0,1])
norm = MidpointNormalize(midpoint=0)
map = ax.contourf(time_18, grid_18*1e3, wxt_18_fil, 400, cmap=plt.cm.bwr, norm=norm, extend='both')
## mark the times for the snapshots ##
ax.plot([time_18[480], time_18[480]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[480], time_18[480]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.51, 0.84, r'$t_{1,b}$', c='k')
ax.plot([time_18[800], time_18[800]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[800], time_18[800]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.84, 0.14, r'$t_{2,b}$', c='k')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.0f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_y(2.3)
    A.set_rotation(70)
##
# ax.set_xlabel(r't [ns]', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{c)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{transition}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.4), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:purple')
at.patch.set_alpha(0.5)
ax.add_artist(at)
##############################################
# Te_18 #
##############################################
ax = fig.add_subplot(pgrid[1,1])
#
map = ax.contourf(time_18, grid_18*1e3, Te_18_fil, 400, cmap=plt.cm.turbo, extend='both')
ax.contour(time_18, grid_18*1e3, Te_18_fil, 7, colors="w", linewidths=0.2, linestyles='dashed')
## mark the times for the snapshots ##
ax.plot([time_18[480], time_18[480]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[480], time_18[480]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.51, 0.84, r'$t_{1,b}$', c='k')
ax.plot([time_18[800], time_18[800]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[800], time_18[800]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.84, 0.14, r'$t_{2,b}$', c='k')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_y(2.3)
    A.set_rotation(70)
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{d)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
#
##############################################
# We_34
##############################################
ax = fig.add_subplot(pgrid[0,2])
norm = MidpointNormalize(midpoint=0)
map = ax.contourf(time_34, grid_34*1e3, wxt_34_fil, 400, cmap=plt.cm.bwr, norm=norm, extend='both')
## mark the times for the snapshots ##
ax.plot([time_34[450], time_34[450]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_34[450], time_34[450]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.49, 0.85, r'$t_{1,c}$', c='k')
ax.plot([time_18[750], time_18[750]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[750], time_18[750]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.78, 0.11, r'$t_{2,c}$', c='k')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.0f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_y(2.3)
    A.set_rotation(70)
## add row label
at = AnchoredText(r'$P_\mathrm{e}$ [$\frac{\mathrm{MW}}{\mathrm{m^3}}$]', frameon=True, loc='center left',#
 bbox_to_anchor=(1.07,.5), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17, rotation=270))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##
# ax.set_xlabel(r't [ns]', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{e)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
## add mode label
at = AnchoredText(r'\textbf{quasi-neutral mode}', frameon=True, loc='lower center',#
 bbox_to_anchor=(0.5,1.4), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('tab:red')
at.patch.set_alpha(0.5)
ax.add_artist(at)
##############################################
# Te_34 #
##############################################
ax = fig.add_subplot(pgrid[1,2])
#
map = ax.contourf(time_34, grid_34*1e3, Te_34_fil, 400, cmap=plt.cm.turbo, extend='both')
ax.contour(time_34, grid_34*1e3, Te_34_fil, 7, colors="w", linewidths=0.2, linestyles='dashed')
## mark the times for the snapshots ##
ax.plot([time_34[450], time_34[450]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_34[450], time_34[450]], [0, 1], c='w', lw=2, ls='-')
ax.text(0.49, 0.85, r'$t_{1,c}$', c='k')
ax.plot([time_18[750], time_18[750]], [0, 1], c='k', lw=3, ls='-')
ax.plot([time_18[750], time_18[750]], [0, 1], c='w', lw=2, ls='--')
ax.text(0.78, 0.11, r'$t_{2,c}$', c='k')
## create colorbar
cbar = plt.colorbar(map, anchor=(1.0,0.0), format="%.1f",#
 location="top", orientation="horizontal", shrink=0.94)
for A in cbar.ax.get_xticklabels():
    A.set_verticalalignment('center')
    A.set_horizontalalignment('center')
    A.set_y(2.3)
    A.set_rotation(70)
## add row label
at = AnchoredText(r'$T_\mathrm{e}$ [$\mathrm{eV}$]', frameon=True, loc='center left',#
 bbox_to_anchor=(1.07,.5), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=17, rotation=270))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)
##
ax.set_xlabel(r't/$T_\mathrm{RF}$', labelpad = 7.0)
# ax.set_ylabel(r'z [mm]', labelpad = 7.0)
ax.tick_params(axis='x', pad = 7.0)
ax.tick_params(axis='y', pad = 7.0)
## add panel label
at = AnchoredText(r'\textbf{f)}', frameon=True, loc='upper left',#
 bbox_to_anchor=(.01,.99), bbox_transform=ax.transAxes, borderpad=0., prop = dict(fontsize=16))
at.patch.set_boxstyle("round,pad=0.,rounding_size=0.3")
at.patch.set_color('lightgrey')
ax.add_artist(at)

##############################################
plt.savefig("fig_9.jpg",format="JPEG",bbox_inches='tight',dpi=300)
# plt.show()
plt.close()
